/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_PROXY_H
#define JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/trpclongservice/trpclongservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace trpclongservice {
namespace proxy {
namespace events {
    using TRPCLong2CANEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::SWC2CAN_Signal_Values>;
    using Long2CANEvent = ara::com::internal::proxy::event::EventAdapter<::SWC2CAN>;
    static constexpr ara::com::internal::EntityId TRPCLongServiceTRPCLong2CANEventId = 20663U; //TRPCLong2CANEvent_event_hash
    static constexpr ara::com::internal::EntityId TRPCLongServiceLong2CANEventId = 59938U; //Long2CANEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class TRPCLongServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              TRPCLong2CANEvent(proxyAdapter->GetProxy(), events::TRPCLongServiceTRPCLong2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              Long2CANEvent(proxyAdapter->GetProxy(), events::TRPCLongServiceLong2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::TRPCLong2CANEvent GetTRPCLong2CANEvent() noexcept { return std::move(TRPCLong2CANEvent); }
        events::Long2CANEvent GetLong2CANEvent() noexcept { return std::move(Long2CANEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::TRPCLong2CANEvent TRPCLong2CANEvent;
        events::Long2CANEvent Long2CANEvent;
    };

    virtual ~TRPCLongServiceProxy()
    {
        TRPCLong2CANEvent.UnsetReceiveHandler();
        TRPCLong2CANEvent.Unsubscribe();
        Long2CANEvent.UnsetReceiveHandler();
        Long2CANEvent.Unsubscribe();

    }

    explicit TRPCLongServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, handle)),
          TRPCLong2CANEvent(proxyAdapter->GetProxy(), events::TRPCLongServiceTRPCLong2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          Long2CANEvent(proxyAdapter->GetProxy(), events::TRPCLongServiceLong2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    TRPCLongServiceProxy(const TRPCLongServiceProxy&) = delete;
    TRPCLongServiceProxy& operator=(const TRPCLongServiceProxy&) = delete;

    TRPCLongServiceProxy(TRPCLongServiceProxy&&) = default;
    TRPCLongServiceProxy& operator=(TRPCLongServiceProxy&&) = default;
    TRPCLongServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          TRPCLong2CANEvent(token.GetTRPCLong2CANEvent()),
          Long2CANEvent(token.GetLong2CANEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<TRPCLongServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::TRPCLong2CANEvent TRPCLong2CANEvent;
    events::Long2CANEvent Long2CANEvent;
};
} // namespace proxy
} // namespace trpclongservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_PROXY_H
